﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Windows.Input;

using Extensibility;

namespace DemoAddin
{
	public class DemoMarkupDialogModel : IDialogContent, ICommand, INotifyPropertyChanged
	{

		public Dictionary<string, object> Values = null;

		private static string[] IssueItems = { 
			"Abandon", "Asset Missing", "Other", "Incorrect Attribute", 
			"Incorrect Location" };
		private static string[] UtilityItems = { 
			"Water", "Sewer", "Stormwater" };
		private static string[] AssetItems_Water = {
			"Casing", "Control Valve", "Curb Stop", "Fitting",
			"Hydrant", "Lateral", "Main", "Network Structure",
			"Pump", "Sample Station", "Service Connection", "System Valve" };
		private static string[] AssetItems_Sewer = { 
			"Casing", "Bend", "Grease Trap", "Clean Out", 
			"Control Valve", "Detention", "Discharge", "Fitting", 
			"Gravity Main", "Inlet", "Lateral", "Manhole",
			"Force Main", "Pump", "Service Connection", "System Valve",
			"Tap", "Test Station", "Valve Operator", "Vault" };
		private static string[] AssetItems_Stormwater = {
			"Casing", "Clean Out", "Control Valve", "Culvert",
			"Detention", "Discharge", "Fitting", "Force Main",
			"Gravity Main", "Inlet", "Manhole", "Open Drain",
			"System Valve", "Weir" };
		private static string[] CriticalItems = { "No", "Yes" };

		private int _iUtilityIndex = -1;
		private List<string> _AssetItems = null;
		private int _iAssetIndex = -1;

		// Bindable properties

		public List<string> IssueItemsSource { get; private set; }

		public int IssueItemsSelectedIndex { get; set; }

		public List<string> UtilityItemsSource { get; private set; }

		public int UtilityItemsSelectedIndex 
		{
			get { return _iUtilityIndex; }
			set 
			{
				_iUtilityIndex = value;
				string[] AssetItems = null;
				switch (_iUtilityIndex)
				{
					case 0:
						AssetItems = AssetItems_Water;
						break;
					case 1:
						AssetItems = AssetItems_Sewer;
						break;
					case 2:
						AssetItems = AssetItems_Stormwater;
						break;
				}
				List<string> NewSource = new List<string>();
				foreach (string sAsset in AssetItems)
					NewSource.Add(sAsset);
				AssetItemsSource = NewSource;
				AssetItemsSelectedIndex = 0;
			} 
		}

		public List<string> AssetItemsSource 
		{
			get { return _AssetItems; }
			private set
			{
				_AssetItems = value;
				this.RaisePropertyChanged();
			}
		}

		public int AssetItemsSelectedIndex 
		{ 
			get { return _iAssetIndex; } 
			set
			{
				_iAssetIndex = value;
				this.RaisePropertyChanged();
			}
		}

		public List<string> CriticalItemsSource { get; private set; }

		public int CriticalItemsSelectedIndex { get; set; }

		public string Notes { get; set; }

		public ICommand Command { get { return this; } }

		// IDialogContent

		public string Title { get; private set; }

		public bool Closable { get { return true; } }

		public event EventHandler<EventArgs> DialogOK;

		public event EventHandler<EventArgs> DialogCancel;

		// ICommand

		public bool CanExecute(object parameter)
		{
			return true;
		}

		public event EventHandler CanExecuteChanged { add { } remove { } } // Not used

		public void Execute(object parameter)
		{

			string sParam = parameter as string;
			switch (sParam)
			{
				case "OK":
					Values = new Dictionary<string, object>();
					Values["STATUS"] = "Open";
					Values["ISSUE"] = IssueItemsSource[IssueItemsSelectedIndex];
					Values["UTILITY"] = (short)UtilityItemsSelectedIndex;
					Values["ASSET"] = AssetItemsSource[AssetItemsSelectedIndex];
					Values["CRITICAL"] = CriticalItemsSource[CriticalItemsSelectedIndex];
					Values["NOTES"] = Notes;
					if (this.DialogOK == null)
						break;
					this.DialogOK(this, new EventArgs());
					break;
				case "Cancel":
					if (this.DialogCancel == null)
						break;
					this.DialogCancel(this, new EventArgs());
					break;
			}

		}

		// INotifyPropertyChanged

		public event PropertyChangedEventHandler PropertyChanged;

		// Constructors

		public DemoMarkupDialogModel()
		{
			this.Init();
		}

		// Private methods

		private void RaisePropertyChanged([CallerMemberName] string sPropertyName = null)
		{
			if (this.PropertyChanged == null)
				return;
			this.PropertyChanged(this, new PropertyChangedEventArgs(sPropertyName));
		}

		private void Init()
		{
			Title = "Map Change Request";
			IssueItemsSource = new List<string>();
			foreach (string sIssue in IssueItems)
				IssueItemsSource.Add(sIssue);
			UtilityItemsSource = new List<string>();
			foreach (string sUtility in UtilityItems)
				UtilityItemsSource.Add(sUtility);
			UtilityItemsSelectedIndex = 0;
			CriticalItemsSource = new List<string>();
			foreach (string sCritical in CriticalItems)
				CriticalItemsSource.Add(sCritical);
			CriticalItemsSelectedIndex = 0;
		}

	}
}