﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Interactivity;

using Esri.ArcGISRuntime.UI.Controls;

using Extensibility;

namespace EditAddin
{

	// EditViewExt is a binding target and adds the following dependency properties:
	//    EditLayersLoadedCommand, ICommand, OneWay

	public class EditViewExt : Behavior<MapView>, IConfigurable
	{

		public static EditViewExt Current { get; private set; }

		public bool LocalGDB { get; set; }

		private string _sConfig;
		private EditHelper _eh;

		// IConfigurable

		public void LoadConfiguration(string sConfigData)
		{
			_sConfig = sConfigData;
		}

		// Behavior

		protected override void OnAttached()
		{
			Current = this;
			base.OnAttached();
			_eh = new EditHelper(AssociatedObject);
			_eh.EditLayersLoaded += EditHelper_EditLayersLoaded;
			_eh.Initialize(_sConfig);
		}

		// EditLayersLoadedCommand property
		// This receives from the binding source an ICommand to execute when edit feature tables are loaded

		public ICommand EditLayersLoadedCommand
		{
			get { return (ICommand)this.GetValue(EditLayersLoadedCommandProperty); }
			set { this.SetValue(EditLayersLoadedCommandProperty, value); }
		}

		public static readonly DependencyProperty EditLayersLoadedCommandProperty =
			DependencyProperty.Register("EditLayersLoadedCommand", typeof(ICommand), typeof(EditViewExt));

		// Public methods

		public void Synchronize(IStatus pStatus)
		{
			Task t = _eh.SynchronizeAsync(pStatus);
		}

		public void DeleteGDB()
		{
			_eh.DeleteGDB();
		}

		// Event handlers

		void EditHelper_EditLayersLoaded(object sender, NotifyLayersLoadedArgs e)
		{
			if (EditLayersLoadedCommand == null)
				return;
			if (!EditLayersLoadedCommand.CanExecute(e.FeatureTables))
				return;
			EditLayersLoadedCommand.Execute(e.FeatureTables);
		}

	}

}
