﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media.Imaging;

using Extensibility;

namespace EditAddin
{

	// CaptureDialog provides support for capturing images

	public class Snapshot
	{
		public string Name { get; set; }
		public string FilePath { get; set; }
	}

	public class CaptureDialogModel : ICommand, IDialogContent
	{

		public Snapshot Snapshot;
		private int _iSelectedIndex = -1;

		// Bindable properties

		public ObservableCollection<FilterInfo> DeviceItems { get; private set; }

		public int DeviceItemsSelectedIndex 
		{
			get { return _iSelectedIndex; }
			set
			{
				_iSelectedIndex = value;
				if (_iSelectedIndex > -1)
				{
					FilterInfo info = DeviceItems[DeviceItemsSelectedIndex];
					CaptureControl.SetDevice(info.MonikerString);
				}
				if (CanExecuteChanged == null)
					return;
				this.CanExecuteChanged(this, new EventArgs());
			}
		}

		public string FileName { get; set; }

		public ICommand Command { get { return this; } }

		public CapPlayer CaptureControl { get; private set; }

		// ICommand

		public bool CanExecute(object parameter) 
		{
			string sParam = parameter as string;
			switch (sParam)
			{
				case "Capture":
					return (DeviceItemsSelectedIndex > -1);
				case "Cancel":
					return true;
			}
			return false;
		}

		public event EventHandler CanExecuteChanged;

		public void Execute(object parameter)
		{
			string sParam = parameter as string;
			switch (sParam)
			{
				case "Capture":
					this.Capture();
					if (this.DialogOK == null)
						break;
					this.DialogOK(this, new EventArgs());
					break;
				case "Cancel":
					if (this.DialogCancel == null)
						break;
					this.DialogCancel(this, new EventArgs());
					break;
				default:
					return;
			}
			CaptureControl.Dispose();
		}

		// IDialogContent

		public string Title
		{
			get { return "Capture Image"; }
		}

		public bool Closable
		{
			get { return true; }
		}

		public event EventHandler<EventArgs> DialogOK;

		public event EventHandler<EventArgs> DialogCancel;

		// Constructor

		public CaptureDialogModel()
		{

			FileName = string.Format("Snapshot_{0:yyyyMMdd_HHmmss}", DateTime.Now);
			DeviceItems = new ObservableCollection<FilterInfo>();
			CaptureControl = new CapPlayer();

			// Get devices

			FilterInfo[] Infos = CapDevice.DeviceMonikers;
			if (Infos.Length == 0)
			{
				MessageBox.Show("No capture devices found");
				return;
			}
			foreach (FilterInfo info in Infos)
				DeviceItems.Add(info);
			DeviceItemsSelectedIndex = 0;

		}

		private void Capture()
		{

			// Get and save capture

			string sSnapFile = Path.GetTempPath() + string.Format("Snapshot_{0:yyyyMMdd_HHmmss}.jpg", DateTime.Now);
			BitmapSource src = CaptureControl.Source as BitmapSource;
			if (src == null)
			{
				MessageBox.Show("Source is null or not BitmapSource");
				return;
			}
			JpegBitmapEncoder jpg = new JpegBitmapEncoder();
			jpg.Frames.Add(BitmapFrame.Create(src));
			using (Stream stm = File.Create(sSnapFile))
			{
				jpg.Save(stm);
			}
			MessageBox.Show("Snapshot captured.");
			Snapshot = new Snapshot();
			Snapshot.Name = FileName;
			Snapshot.FilePath = sSnapFile;

		}


	}

}
