﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Xml.Linq;

using Esri.ArcGISRuntime.Data;

using Extensibility;

namespace EditAddin
{

	// EditModelExt hosts feature editing and synchronization capabilities

	public class EditModelExt : IConfigurable
	{

		public static EditModelExt Current { get; private set; }
		public Dictionary<string, ArcGISFeatureTable> FeatureTables;

		// Layout bindable properties

		public Visibility SynchronizeVisibility { get; private set; }

		public ICommand SynchronizeCommand { get; private set; }

		public ICommand DeleteGDBCommand { get; private set; }

		// View bindable properties

		public ICommand ViewEditLayersLoadedCommand { get; private set; }

		// IConfigurable

		public void LoadConfiguration(string sConfigData)
		{

			// Synchronize is visible by default

			if (string.IsNullOrEmpty(sConfigData))
				return;
			XDocument xDoc = XDocument.Parse(sConfigData);
			XElement xRoot = xDoc.Root;
			XElement xElem = xRoot.Element("SynchronizeVisible");
			if (xElem != null)
			{
				bool bNotVis = xElem.Value == "False";
				if (bNotVis)
					SynchronizeVisibility = Visibility.Collapsed;
			}
		}

		// Public events

		public event EventHandler EditDataLoaded;
		public event EventHandler<NotifyCanExecuteArgs> CanSynchronize;

		// Constructor

		public EditModelExt()
		{
			Current = this;
			SynchronizeVisibility = Visibility.Visible;
			CommandHelper ch;
			ch = new CommandHelper(false);
			ch.AddExecuteAction(this.ExecuteSynchronize);
			ch.AddCanExecuteEvent(this, "CanSynchronize");
			SynchronizeCommand = ch;
			ch = new CommandHelper(false);
			ch.AddExecuteAction(this.ExecuteDeleteGDB);
			ch.AddCanExecuteEvent(this, "CanSynchronize");
			DeleteGDBCommand = ch;
			ch = new CommandHelper(true);
			ch.AddExecuteAction(this.ExecuteEditLayersLoaded);
			ViewEditLayersLoadedCommand = ch;
			AppViewExt.Current.MapChanged += MapApplication_MapChanged;
		}

		// Public methods

		public ArcGISFeatureTable GetFeatureTable(string sTableName)
		{
			bool bSpace = false;
			string sGDBName = null;
			if (EditViewExt.Current.LocalGDB && sTableName.Contains(" "))
			{
				// A GDB table name will not contain spaces
				sGDBName = sTableName.Replace(" ", "_");
				bSpace = true;
			}
			foreach (string sKey in FeatureTables.Keys)
			{
				ArcGISFeatureTable fTab = FeatureTables[sKey];
				string sTabName = fTab.TableName;
				if (sTabName == sTableName)
					return fTab;
				else if (bSpace && (sTabName == sGDBName))
					return fTab;
			}
			return null;
		}

		// Event handlers

		void MapApplication_MapChanged(object sender, MapChangedArgs e)
		{
			if (this.CanSynchronize == null)
				return; // This should never happen
			NotifyCanExecuteArgs args = new NotifyCanExecuteArgs();
			args.CanExecute = true;
			this.CanSynchronize(this, args);
		}

		// Private methods

		private void ExecuteEditLayersLoaded(object parameter)
		{
			FeatureTables = parameter as Dictionary<string, ArcGISFeatureTable>;
			if (this.EditDataLoaded == null)
				return;
			this.EditDataLoaded(this, new EventArgs());
		}

		private void ExecuteSynchronize(object parameter)
		{
			SynchronizeDialogModel.Open();
		}

		private void ExecuteDeleteGDB(object parameter)
		{
			EditViewExt.Current.DeleteGDB();
		}

	}

}
