﻿using System;
using System.IO;
using System.Xml.Linq;

namespace Extensibility
{

	// SettingsHelper provides a central management of settings in the form of xml documents.
	// This allows extensions to store and retrieve their own session data.

	public static class SettingsHelper
	{

		public static XDocument OpenSettings(string sFileName)
		{
			XDocument xSettings = null;
			string sUserSettingsPath = SettingsHelper.GetUserSettingsPath();
			string sUserSettingsFile = sUserSettingsPath + "\\" + sFileName;
			if (!File.Exists(sUserSettingsFile))
			{
				xSettings = new XDocument();
				XElement xRoot = new XElement("Settings");
				xSettings.Add(xRoot);
			}
			else
			{
				StreamReader sr = new StreamReader(sUserSettingsFile);
				xSettings = XDocument.Load(sr);
				sr.Close();
			}
			return xSettings;
		}

		public static void SaveSettings(string sFileName, XDocument xSettings)
		{
			string sUserSettingsPath = SettingsHelper.GetUserSettingsPath();
			string sUserSettingsFile = sUserSettingsPath + "\\" + sFileName;
			StreamWriter sw = new StreamWriter(sUserSettingsFile);
			xSettings.Save(sw);
			sw.Close();
		}

		public static string GetUserSettingsPath()
		{
			string sUserSettingsPath = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
			sUserSettingsPath += "\\" + MapApplication.Current.ApplicationName;
			if (!Directory.Exists(sUserSettingsPath))
				Directory.CreateDirectory(sUserSettingsPath);
			return sUserSettingsPath;
		}
		
	}

}
