﻿using System;
using System.Windows.Controls;
using System.Windows.Media;

namespace Extensibility
{

	// IMainWindow must be implemented by the main window class of the application
	public interface IMainWindow
	{
		// ApplicationName is an arbitrary name, not necessarily that of the executable
		string ApplicationName { get; }
		// WindowIcon is the default icon to use for child windows
		ImageSource WindowIcon { get; }
		// RuntimeVersion is the ArcGIS Runtime version number (e.g. 100.7)
		string RuntimeVersion { get; }
		// UpdateLocation is an optional folder to check for updates
		string UpdateLocation { get; }
		// Configuration is the keyword used for loading the configuration file
		string Configuration { get; }
		// SetTitle sets the title of the main window
		void SetTitle(string sTitle);
		// SetLayout provides the user control representing the UI to be used by the main window
		void SetLayout(UserControl ucLayout);
		// Closing notifies that the application is closing and should be cleaned up
		event EventHandler AppClosing;
	}

	// IStatus may be implemented by a model extension
	// Otherwise, MapApplication will use a default implementation
	public interface IStatus
	{
		void Status(string sMsg);
		void Status(string sMsg, bool bBusy);
		void Message(string sMsg);
		void Error(string sErr);
	}

	// IConfigurable is optionally implemented by model or view extensions
	public interface IConfigurable
	{
		void LoadConfiguration(string sConfigData);
	}

	// IWindowContent is implemented by the view model for a nonmodal dialog window
	public interface IWindowContent
	{
		// Title is bound to the child window's title
		string Title { get; } 
		event EventHandler<EventArgs> ShowWindow;
		event EventHandler<EventArgs> HideWindow;
		void ShutDown(); // Perform cleanup at application close
	}

	// IDialogContent is implemented by the view model for a modal dialog window
	public interface IDialogContent
	{
		// Title is bound to the window's title
		string Title { get; }
		// If Closable is false, it disables the window close button
		bool Closable { get; } 
		event EventHandler<EventArgs> DialogOK;
		event EventHandler<EventArgs> DialogCancel;
	}

	// IMapTool is implemented by tools that are registered with AppViewExt
	public interface IMapTool
	{
		// Tool ID should be a unique string, typically Guid.NewGuid().ToString()
		string ID { get; }
		ToolConfiguration Configuration { get; }
		void Activate();
		void Deactivate();
		void NotifyProgress(Esri.ArcGISRuntime.Geometry.Geometry g);
		void NotifyComplete(Esri.ArcGISRuntime.Geometry.Geometry g);
	}

}
