﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

namespace Extensibility
{

	public class ChildItem : INotifyPropertyChanged
	{

		private bool _bIsSelected = false;
		private Visibility _visItem = Visibility.Visible;

		// Bindable properties

		public UserControl Content { get; private set; }

		public object Model { get; private set; }

		public ImageSource Icon { get; private set; }

		public Visibility IconVisibility { get; private set; }

		public string Title { get; private set; }

		public Visibility TitleVisibility { get; private set; }

		public bool IsSelected
		{
			get { return _bIsSelected; }
			set
			{
				_bIsSelected = value;
				this.RaisePropertyChanged();
			}
		}

		public Visibility Visibility
		{
			get { return _visItem; }
			set
			{
				_visItem = value;
				this.RaisePropertyChanged();
			}
		}

		public ICommand CloseCommand { get; private set; }		

		// INotifyPropertyChanged

		public event PropertyChangedEventHandler PropertyChanged;

		// Public methods

		public void SetContent(UserControl MyView, IWindowContent MyViewModel)
		{

			MyView.DataContext = MyViewModel;
			Content = MyView;
			Model = MyViewModel;
			Title = MyViewModel.Title;
			if (MapApplication.Current.ShowChildItemTitle && (!string.IsNullOrEmpty(MyViewModel.Title)))
				TitleVisibility = Visibility.Visible;
			MyViewModel.ShowWindow += MyViewModel_ShowWindow;
			MyViewModel.HideWindow += MyViewModel_HideWindow;

			// Get icon if one exists

			ResourceDictionary rd = MyView.Resources;
			if (MapApplication.Current.ShowChildItemIcon && rd.Contains("Icon"))
			{
				Image img = rd["Icon"] as Image;
				if (img != null)
				{
					Icon = img.Source;
					IconVisibility = Visibility.Visible;
				}
			}

		}

		public void ShutDown()
		{
			IWindowContent MyViewModel = (IWindowContent)Model;
			MyViewModel.ShutDown();
		}

		// Constructor

		public ChildItem()
		{
			Visibility = Visibility.Visible;
			IconVisibility = Visibility.Collapsed;
			TitleVisibility = Visibility.Collapsed;
			CommandHelper ch = new CommandHelper(true);
			ch.AddExecuteAction(this.ExecuteHide);
			CloseCommand = ch;
		}

		// Event handlers

		private void MyViewModel_HideWindow(object sender, EventArgs e)
		{
			this.ExecuteHide(null);
		}

		private void MyViewModel_ShowWindow(object sender, EventArgs e)
		{
			this.ExecuteShow(null);
		}

		// Private methods

		private void RaisePropertyChanged([CallerMemberName]string sPropertyName = null)
		{
			if (this.PropertyChanged == null)
				return;
			this.PropertyChanged(this, new PropertyChangedEventArgs(sPropertyName));
		}

		private void ExecuteHide(object parameter)
		{
			IsSelected = false;
			Visibility = Visibility.Collapsed;
			this.SelectNext();
			MapApplication.Current.MainLayoutModel.CheckChildItemsVisibility();
		}

		private void ExecuteShow(object parameter)
		{
			Visibility = Visibility.Visible;
			IsSelected = true;
			MapApplication.Current.MainLayoutModel.ChildItemsVisible = true;
		}

		private void SelectNext()
		{

			int iCount = MapApplication.Current.MainLayoutModel.ChildItems.Count;
			int iPos = MapApplication.Current.MainLayoutModel.ChildItems.IndexOf(this);

			// Find next visible item

			for (int i = iPos + 1; i < iCount; i++)
			{
				ChildItem item = MapApplication.Current.MainLayoutModel.ChildItems[i];
				if (item.Visibility != Visibility.Visible)
					continue;
				item.IsSelected = true;
				return;
			}

			// Find previous visible item

			for (int i = iPos - 1; i > -1; i--)
			{
				ChildItem item = MapApplication.Current.MainLayoutModel.ChildItems[i];
				if (item.Visibility != Visibility.Visible)
					continue;
				item.IsSelected = true;
				return;
			}

		}

	}

}
