﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;

using Esri.ArcGISRuntime.UI.Controls;

namespace Extensibility
{

	// MainLayoutModel provides the master data context for the layout.
	// Additional data contexts are added via the DataContexts property.

	public class MainLayoutModel : DependencyObject, INotifyPropertyChanged
	{

		private DataHelper _Data;
		private MapView _mv;
		private bool _bChildItemsVisible = false;

		// Layout bindable properties

		public IDictionary<string, object> DataContexts { get { return _Data; } }

		public MapView MapViewContainerContent
		{
			get { return _mv; }
			internal set
			{
				_mv = value;
				this.RaisePropertyChanged();
			}
		}

		public ObservableCollection<ChildItem> ChildItems { get; private set; }

		public bool ChildItemsVisible
		{
			get { return _bChildItemsVisible; }
			set
			{
				_bChildItemsVisible = value;
				this.RaisePropertyChanged();
			}
		}

		// INotifyPropertyChanged

		public event PropertyChangedEventHandler PropertyChanged;

		// Public methods

		public void CheckChildItemsVisibility()
		{

			// If all items are hidden, hide collection

			bool bHide = true;
			foreach (ChildItem item in ChildItems)
			{
				if (item.Visibility == Visibility.Visible)
				{
					bHide = false;
					break;
				}
			}
			if (!bHide)
				return;
			ChildItemsVisible = false;

		}

		// Constructor

		public MainLayoutModel()
		{
			_Data = new DataHelper();
			_Data.PropertyChanged += Data_PropertyChanged;
			ChildItems = new ObservableCollection<ChildItem>();
		}

		// Event handlers

		private void Data_PropertyChanged(object sender, PropertyChangedEventArgs e)
		{
			this.RaisePropertyChanged("DataContexts");
		}

		// Private methods

		private void RaisePropertyChanged([CallerMemberName]string sPropertyName = null)
		{
			if (this.PropertyChanged == null)
				return;
			this.PropertyChanged(this, new PropertyChangedEventArgs(sPropertyName));
		}

	}

}
