﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;
using System.Xml.Linq;

using Extensibility;

namespace LayoutAddin
{

	// LayoutDialog is the view model for the layout dialog window

	class LayoutDialogModel : INotifyPropertyChanged, IDialogContent, ICommand
	{

		private Dictionary<string, Layout> _Layouts;
		private int _iLayoutIndex;

		// Bindable properties

		public ObservableCollection<string> LayoutItemsSource { get; private set; }

		public int LayoutSelectedIndex
		{
			get { return _iLayoutIndex; }
			set
			{
				_iLayoutIndex = value;
				this.SelectLayout();
				this.RaisePropertyChanged();
			}
		}

		public ObservableCollection<LayoutProperty> PropertyItemsSource { get; private set; }

		public ICommand Command { get { return this; } }

		// INotifyPropertyChanged

		public event PropertyChangedEventHandler PropertyChanged;

		// IDialogContent

		public string Title
		{
			get { return "Open Layout"; }
		}

		public bool Closable
		{
			get { return true; }
		}

		public event EventHandler<EventArgs> DialogOK;

		public event EventHandler<EventArgs> DialogCancel;

		// ICommand

		public bool CanExecute(object parameter)
		{
			return true;
		}

		public event EventHandler CanExecuteChanged { add { } remove { } } // Not used

		public void Execute(object parameter)
		{
			string sParam = parameter as string;
			if (sParam == null)
				return;
			switch (sParam)
			{
				case "OK":
					this.OK();
					break;
				case "Cancel":
					this.Cancel();
					break;
			}
		}

		// Constructors

		public LayoutDialogModel(List<Layout> Layouts)
		{

			_Layouts = new Dictionary<string,Layout>();
			LayoutItemsSource = new ObservableCollection<string>();
			PropertyItemsSource = new ObservableCollection<LayoutProperty>();

			// Populate layout items

			List<string> LayoutList = new List<string>();
			foreach (Layout lo in Layouts)
			{
				string sKey = lo.DisplayName;
				LayoutList.Add(sKey);
				_Layouts[sKey] = lo;
			}
			foreach (string sVal in LayoutList)
				LayoutItemsSource.Add(sVal);
			LayoutSelectedIndex = 0;

		}

		// Private methods

		private void RaisePropertyChanged([CallerMemberName]string sPropertyName = null)
		{
			if (this.PropertyChanged == null)
				return;
			this.PropertyChanged(this, new PropertyChangedEventArgs(sPropertyName));
		}

		private void SelectLayout()
		{

			// Populate property items

			string sLayout = LayoutItemsSource[LayoutSelectedIndex];
			List<LayoutProperty> PropertyList = _Layouts[sLayout].Properties;
			PropertyItemsSource.Clear();
			if (PropertyList == null)
				return;
			foreach (LayoutProperty prop in PropertyList)
			{
				if (prop.Type != LayoutPropertyType.Input)
					continue;
				PropertyItemsSource.Add(prop);
			}

		}

		private void OK()
		{

			// Set up parameters and call open layout command

			string sLayout = LayoutItemsSource[LayoutSelectedIndex];
			Layout lo = _Layouts[sLayout];
			LayoutViewExt.Current.OpenLayout(lo);

			// Close dialog

			if (this.DialogOK == null)
				return;
			this.DialogOK(this, new EventArgs());

		}

		private void Cancel()
		{
			if (this.DialogCancel == null)
				return;
			this.DialogCancel(this, new EventArgs());
		}

	}

}
