﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Xml.Linq;

using Extensibility;
using NavAddin;

namespace LayoutAddin
{

	// LayoutModelExt manages layout display

	public enum LayoutPropertyType
	{
		Unknown,
		Input,
		Date,
		Scale
	}

	public class LayoutProperty
	{
		public LayoutPropertyType Type = LayoutPropertyType.Unknown;
		public string Name { get; set; }
		public string Label { get; set; }
		public string Value { get; set; }
		public Dictionary<string, string> Parameters = new Dictionary<string, string>();
	}

	public class Layout
	{
		public string Name { get; set; }
		public string DisplayName { get; set; }
		public double PageWidth { get; set; }
		public double PageHeight { get; set; }
		public List<LayoutProperty> Properties = new List<LayoutProperty>();
	}

	public class LayoutModelExt : IConfigurable
	{

		public static LayoutModelExt Current { get; private set; }

		private List<Layout> _Layouts;

		// Bindable main layout properties

		public ICommand OpenLayoutCommand { get; private set; }

		public Visibility OpenLayoutVisibility { get; private set; }

		// IConfigurable

		public void LoadConfiguration(string sConfigData)
		{
			_Layouts = new List<Layout>();
			if (string.IsNullOrEmpty(sConfigData))
				return;
			XDocument xDoc = XDocument.Parse(sConfigData);
			XElement xRoot = xDoc.Root;
			XElement xLayouts = xRoot.Element("Layouts");
			foreach (XElement xLayout in xLayouts.Elements("Layout"))
			{

				// Get attributes

				Layout lo = new Layout();
				lo.Name = xLayout.Attribute("Name").Value;
				lo.DisplayName = xLayout.Attribute("DisplayName").Value;
				lo.PageWidth = Convert.ToDouble(xLayout.Attribute("PageWidth").Value);
				lo.PageHeight = Convert.ToDouble(xLayout.Attribute("PageHeight").Value);
				_Layouts.Add(lo);

				// Get bindable properties, if any

				XElement xProperties = xLayout.Element("Properties");
				if (xProperties == null)
					continue;
				foreach (XElement xProperty in xProperties.Elements("Property"))
				{
					LayoutProperty prop = new LayoutProperty();
					foreach (XAttribute xAtt in xProperty.Attributes())
					{
						string sAttrName = xAtt.Name.LocalName;
						string sAttrVal = xAtt.Value;
						switch (sAttrName)
						{
							case "Name":
								prop.Name = sAttrVal;
								break;
							case "Label":
								prop.Label = sAttrVal;
								break;
							case "Type":
								switch (sAttrVal)
								{
									case "Input":
										prop.Type = LayoutPropertyType.Input;
										break;
									case "Date":
										prop.Type = LayoutPropertyType.Date;
										break;
									case "Scale":
										prop.Type = LayoutPropertyType.Scale;
										break;
								}
								break;
							default:
								prop.Parameters[sAttrName] = sAttrVal;
								break;
						}
					} // foreach xAtt
					if (prop.Type == LayoutPropertyType.Unknown)
						continue;
					lo.Properties.Add(prop);
				} // foreach xProperty

			} // foreach xLayout
		}

		// Constructors

		public LayoutModelExt()
		{
			Current = this;
			CommandHelper ch = new CommandHelper(false);
			ch.AddExecuteAction(this.ExecuteOpenLayout);
			ch.AddCanExecuteEvent(NavModelExt.Current, "CanExecuteNavCommand");
			OpenLayoutCommand = ch;
			OpenLayoutVisibility = Visibility.Visible;
		}

		// Private methods

		private void ExecuteOpenLayout(object parameter)
		{

			// Open layout dialog

			LayoutDialogModel ldm = new LayoutDialogModel(_Layouts);
			MapApplication.Current.CreateChildWindow("LayoutAddin", "LayoutDialog", ldm);

		}

	}
}
