﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Esri.ArcGISRuntime.Mapping;

namespace NavAddin
{

	/*
	 * **** RUNTIME 100.6 WORKAROUND ****
	 * While 100.6 properly hydrates group layers in a mobile map package,
	 * the GroupLayer class has severe bugs associated with it.
	 * MobileMapHelper extracts non-group layers and analyzes layer organization so that
	 * MapHelper can construct a TOC item that imitates group layers
	*/

	public class MobileLayerTree : ILayerContent
	{

		public string LayerID { get; set; }
		public List<MobileLayerTree> Sublayers = new List<MobileLayerTree>();
		public double MinScale { get; set; }
		public double MaxScale { get; set; }

		// ILayerContent

		public bool CanChangeVisibility { get { return true; } }
		public bool IsVisible { get; set; }
		public string Name { get; set; }
		public bool ShowInLegend { get; set; }
		public IReadOnlyList<ILayerContent> SublayerContents { get { return Sublayers.AsReadOnly(); } }
		public Task<IReadOnlyList<LegendInfo>> GetLegendInfosAsync()
		{
			return null; 
		}
		public bool IsVisibleAtScale(double scale)
		{
			return IsVisible;
		}

	}

	public struct MapResult
	{
		public List<Layer> Layers;
		public List<MobileLayerTree> TreeEntries;
	}

	public static class MobileMapHelper
	{

		private static List<Layer> _layers;
		private static List<MobileLayerTree> _entries;
		private static Dictionary<string, Layer> _OpLayers;

		public static MapResult ParseMap(Map map)
		{
			_layers = new List<Layer>();
			_entries = new List<MobileLayerTree>();
			foreach (Layer lyr in map.OperationalLayers.ToList())
			{
				map.OperationalLayers.Remove(lyr);
				ProcessLayer(lyr, null);
			}
			MapResult result = new MapResult()
			{
				Layers = _layers,
				TreeEntries = _entries
			};
			_layers = null;
			_entries = null;
			return result;
		}

		public static async Task<List<TocDataItem>> GetTOC(MobileLayerTree mltMap, Dictionary<string, Layer> Layers, int iTocExpansionLevel)
		{
			_OpLayers = Layers;
			List<TocDataItem> Items = await GetTOC(mltMap.Sublayers, iTocExpansionLevel, 0);
			_OpLayers = null;
			return Items;
		}

		private static void ProcessLayer(Layer lyr, MobileLayerTree parent)
		{
			MobileLayerTree entry = new MobileLayerTree()
			{
				LayerID = null,
				Name = lyr.Name,
				MinScale = lyr.MinScale,
				MaxScale = lyr.MaxScale,
				IsVisible = lyr.IsVisible,
				ShowInLegend = lyr.ShowInLegend,
			};
			if (parent == null)
				_entries.Add(entry);
			else
				parent.Sublayers.Add(entry);
			if (!(lyr is GroupLayer grpLayer))
			{
				entry.LayerID = lyr.Id;
				_layers.Add(lyr);
				return;
			}
			foreach (Layer lyrChild in grpLayer.Layers.ToList())
			{
				grpLayer.Layers.Remove(lyrChild);
				ProcessLayer(lyrChild, entry);
			}
		}

		private static async Task<List<TocDataItem>> GetTOC(List<MobileLayerTree> sublayers, int iExpansionLevel, int iCurrentLevel)
		{

			List<TocDataItem> Items = new List<TocDataItem>();
			foreach (MobileLayerTree sublayer in sublayers)
			{
				TocDataItem item = new TocDataItem(true);
				if (!string.IsNullOrEmpty(sublayer.LayerID))
				{
					if (_OpLayers.ContainsKey(sublayer.LayerID))
					{
						Layer lyr = _OpLayers[sublayer.LayerID];
						await item.SetLayer(lyr, iExpansionLevel, iCurrentLevel);
					}
				}
				else
				{
					item.DisplayName = sublayer.Name;
					item.IsChecked = sublayer.IsVisible;
					item.MinScale = sublayer.MinScale;
					item.MaxScale = sublayer.MaxScale;
					List<TocDataItem> Subitems = await GetTOC(sublayer.Sublayers, iExpansionLevel, iCurrentLevel + 1);
					item.SetSubitems(Subitems);
				}
				if (iCurrentLevel <= iExpansionLevel)
					item.IsExpanded = true;
				Items.Add(item);
			}
			Items.Reverse();
			return Items;

		}


	}
}
