﻿using System.Collections.ObjectModel;
using System.Linq;

namespace NavAddin
{

	// ScaleHelper helps manage and navigate the scale list

	public class ScaleHelper
	{

		public ObservableCollection<string> ScaleItems { get; private set; }

		private string[] _Scales = { "0:0", "1:1,000", "1:10,000", "1:24,000", "1:100,000", "1:250,000" };

		public ScaleHelper()
		{

			// Populate scale items

			ScaleItems = new ObservableCollection<string>();
			foreach (string sScale in _Scales)
				ScaleItems.Add(sScale);

		}

		public bool SetScale(double dNewScale)
		{
			if (double.IsNaN(dNewScale))
				return false;
			if (dNewScale <= 0)
				return false;
			string sNewScale = "1:" + dNewScale.ToString("N0");
			string sOldScale = ScaleItems[0].ToString();
			if (sNewScale == sOldScale)
				return false;
			ScaleItems[0] = sNewScale;
			return true;
		}

		public double GetScale(string sNewScale)
		{
			if (string.IsNullOrEmpty(sNewScale))
				return double.NaN;
			if (sNewScale.Contains(':'))
			{
				string[] Tok = sNewScale.Split(':');
				sNewScale = Tok[1];
			}
			sNewScale = sNewScale.Replace(",", "");
			double dNewScale = 0;
			if (!double.TryParse(sNewScale, out dNewScale))
				return double.NaN;
			return dNewScale;
		}

		public double GetScale(int iIndex)
		{
			if (iIndex < 0)
				return double.NaN;
			if (iIndex >= ScaleItems.Count)
				return double.NaN;
			return this.GetScale(ScaleItems[iIndex]);
		}


	}

}
