﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace NavAddin
{

	/*
	 * Runtime 100.5/100.6 BUG: Tile caches that have only one scale level level defined will hang up on loading.
	 * WORKAROUND: Define additional scale levels
	 * [Assumes that 100 < scale level < 24000]
	*/

	public static class TileCacheHelper
	{

		public const string L0Scale = "24000";
		public const string L0Resolution = "20.833375000083333";
		public const string L2Scale = "100";
		public const string L2Resolution = "0.086805729167013887";

		public static bool CheckTileCache(string sPath)
		{

			// Check if tile cache (i.e. a folder)

			if (!Directory.Exists(sPath))
				return true; // Not a tile cache

			// Check if one scale level defined

			string sConfigPath = Path.Combine(sPath, "conf.xml");
			StreamReader sr = new StreamReader(sConfigPath);
			XDocument xDoc = XDocument.Load(sr);
			sr.Close();
			XElement xRoot = xDoc.Root;
			XElement xTCInfo = xRoot.Element("TileCacheInfo");
			XElement xLODInfos = xTCInfo.Element("LODInfos");
			int iLevelCount = xLODInfos.Elements("LODInfo").Count();
			if (iLevelCount > 1)
				return true; // Not a problem
			if (iLevelCount < 1)
				return false; // This should never happen?

			// Check if scale level is between 100 (L2) and 24000 (L0)

			XElement xLODInfo, xLevelID, xScale, xResolution;

			xLODInfo = xLODInfos.Element("LODInfo");
			xScale = xLODInfo.Element("Scale");
			string sScale = xScale.Value;
			double dScale = Convert.ToDouble(sScale);
			double dL0Scale = Convert.ToDouble(L0Scale);
			double dL2Scale = Convert.ToDouble(L2Scale);
			if (dScale >= dL0Scale)
				return false;
			if (dScale <= dL2Scale)
				return false;

			// Redefine scale levels

			xLevelID = xLODInfo.Element("LevelID");
			xLevelID.Value = "1";
			XElement xLOD0 = new XElement(xLODInfo);
			xLevelID = xLOD0.Element("LevelID");
			xLevelID.Value = "0";
			xScale = xLOD0.Element("Scale");
			xScale.Value = L0Scale;
			xResolution = xLOD0.Element("Resolution");
			xResolution.Value = L0Resolution;
			xLODInfos.AddFirst(xLOD0);
			XElement xLOD2 = new XElement(xLODInfo);
			xLevelID = xLOD2.Element("LevelID");
			xLevelID.Value = "2";
			xScale = xLOD2.Element("Scale");
			xScale.Value = L2Scale;
			xResolution = xLOD2.Element("Resolution");
			xResolution.Value = L2Resolution;
			xLODInfos.Add(xLOD2);

			// Write config file

			StreamWriter sw = new StreamWriter(sConfigPath);
			xDoc.Save(sw);
			sw.Close();

			// Rename L00 folder to L01

			string sLayersPath = Path.Combine(sPath, "_alllayers");
			string sL00Path = Path.Combine(sLayersPath, "L00");
			string sL01Path = Path.Combine(sLayersPath, "L01");
			Directory.Move(sL00Path, sL01Path);

			return true;

		}



	}
}
