﻿using System;
using System.Collections.Generic;
using System.Text;

using Esri.ArcGISRuntime.Geometry;

namespace QueryAddin
{

	public static class GeometryHelper
	{

		// Public methods

		//public static double Distance(MapPoint pt1, MapPoint pt2)
		//{
		//	double dX = pt2.X - pt1.X;
		//	double dY = pt2.Y - pt1.Y;
		//	return Math.Sqrt((dX * dX) + (dY * dY));
		//}

		//public static double Length(Segment seg)
		//{
		//	return GeometryHelper.Distance(seg.StartPoint, seg.EndPoint);
		//}

		public static string Bearing(Segment seg)
		{
			MapPoint pt1 = seg.StartPoint;
			MapPoint pt2 = seg.EndPoint;
			StringBuilder sb = new StringBuilder();
			double dX = pt2.X - pt1.X;
			double dY = pt2.Y - pt1.Y;
			int iAngle = (int)AngularUnits.Degrees.FromRadians(Math.Atan2(dY, dX));
			if (iAngle < 0)
			{
				iAngle = Math.Abs(iAngle);
				sb.Append("S");
			}
			else
				sb.Append("N");
			if (iAngle > 90)
			{
				iAngle -= 90;
				sb.Append(" " + iAngle.ToString() + "° W");
			}
			else if (iAngle < 90)
			{
				iAngle = 90 - iAngle;
				sb.Append(" " + iAngle.ToString() + "° E");
			}
			return sb.ToString();
		}

		public static MapPoint GetCenter(Geometry g)
		{
			if (g == null)
				return null;
			if (g.IsEmpty)
				return null;
			if (g is Polyline)
			{
				Polyline p = (Polyline)g;
				if (p.HasCurves)
					p = (Polyline)GeometryEngine.Generalize(p, 1.0, true);
				double d = GeometryEngine.Length(p) / 2;
				return GeometryEngine.CreatePointAlong(p, d);
			}
			else if (g is Polygon)
			{
				Polygon p = (Polygon)g;
				if (p.HasCurves)
					p = (Polygon)GeometryEngine.Generalize(p, 1.0, true);
				return GeometryEngine.LabelPoint(p);
			}
			return g.Extent.GetCenter();
		}

		public static string ToDMS(double d, bool bIsLongitude)
		{
			int iD = (int)Math.Floor(Math.Abs(d));
			string sDegrees = iD.ToString() + "°";
			double dMS = (Math.Abs(d) - (double)iD) * 60;
			int iM = (int)Math.Floor(Math.Abs(dMS));
			sDegrees += iM.ToString() + "'";
			double dS = (dMS - iM) * 60;
			sDegrees += dS.ToString("N3") + '"';
			if (bIsLongitude)
			{
				if (d < 0)
					sDegrees += "W";
				else
					sDegrees += "E";
			}
			else
			{
				if (d < 0)
					sDegrees += "S";
				else
					sDegrees += "N";
			}
			return sDegrees;
		}

		public static double FromDMS(string sDegrees)
		{
			double dDegrees = 0;
			string sEntry = sDegrees.ToUpper();
			bool bNeg = (sEntry.Contains("W") || sEntry.Contains("S"));
			char[] seps = { '°', '\'', '"', 'W', 'E', 'S', 'N', ' ' };
			string[] sTokens = sEntry.Split(seps);
			int i = 0;
			foreach (string sToken in sTokens)
			{
				if (sToken == "")
					continue;
				double d;
				bool bResult = double.TryParse(sToken, out d);
				if (!bResult)
					return double.NaN;
				switch (i)
				{
					case 0:
						dDegrees += d;
						break;
					case 1:
						dDegrees += d / 60;
						break;
					case 2:
						dDegrees += d / 3600;
						break;
				}
				i++;
			}
			if (bNeg)
				dDegrees = -dDegrees;
			return dDegrees;
		}

		// Private methods

		//private static MapPoint Along(Polyline Line, double dDistance, bool bAsRatio)
		//{

		//	// Return a point representing a measurement along a polyline

		//	MapPoint ptAlong = null;
		//	try
		//	{
		//		List<LineSegment> SegList = new List<LineSegment>();
		//		List<double> LenList = new List<double>();
		//		double dTotalLen = 0;
		//		int iCount = 0;

		//		foreach (ReadOnlyPart p in Line.Parts)
		//		{
		//			foreach (LineSegment seg in p)
		//			{
		//				double dLen = GeometryHelper.Length(seg);
		//				SegList.Add(seg);
		//				LenList.Add(dLen);
		//				dTotalLen += dLen;
		//				iCount++;
		//			}
		//		}

		//		double dQueryDistance = dDistance;
		//		if (bAsRatio)
		//			dQueryDistance *= dTotalLen;
		//		dTotalLen = 0;
		//		for (int i = 0; i < iCount; i++)
		//		{
		//			double dLen = LenList[i];
		//			dTotalLen += dLen;
		//			if ((dTotalLen < dQueryDistance) && (i < iCount - 1))
		//				continue;
		//			double d = dLen - (dTotalLen - dQueryDistance);
		//			LineSegment seg = SegList[i];
		//			ptAlong = GeometryHelper.Along(seg, d, false);
		//			break;
		//		}

		//	}
		//	catch
		//	{
		//		ptAlong = null;
		//	}
		//	return ptAlong;

		//}

		//private static MapPoint Along(LineSegment seg, double dDistance, bool bAsRatio)
		//{
		//	MapPoint ptFrom = seg.StartPoint;
		//	MapPoint ptTo = seg.EndPoint;
		//	double d = dDistance;
		//	if (!bAsRatio)
		//		d /= GeometryHelper.Length(seg);
		//	double dX = ptFrom.X + ((ptTo.X - ptFrom.X) * d);
		//	double dY = ptFrom.Y + ((ptTo.Y - ptFrom.Y) * d);
		//	return new MapPoint(dX, dY, seg.SpatialReference);
		//}

	}

}
