﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows.Input;

namespace QueryAddin
{

	// IdentifyListHelper provides view model support for navigating 
	// a list-style collection of identify results

	class IdentifyListHelper : ICommand
	{

		private ObservableCollection<IdentifyResult> _Results;
		private List<IdentifyResult> _ResultList;
		private bool _bTree;
		private int _iIndex = -1;

		// ICommand

		public bool CanExecute(object parameter)
		{
			if (_ResultList.Count < 2)
				return false;
			string sParam = parameter as string;
			switch (sParam)
			{
				case "First":
				case "Previous":
					return (_iIndex > 0);
				case "Next":
				case "Last":
					return (_iIndex < _Results.Count - 1);
			}
			return false;
		}

		public event EventHandler CanExecuteChanged;

		public void Execute(object parameter)
		{
			int i = _iIndex;
			if (i > -1)
				_ResultList[i].IsSelected = false;
			string sParam = parameter as string;
			switch (sParam)
			{
				case "First":
					i = 0;
					break;
				case "Previous":
					if (i > 0)
						i--;
					break;
				case "Next":
					if (i < _ResultList.Count - 1)
						i++;
					break;
				case "Last":
					i = _ResultList.Count - 1;
					break;
			}
			_ResultList[i].IsSelected = true;
		}

		// Public methods

		public void Initialize(ObservableCollection<IdentifyResult> ResultsList, bool bTree)
		{
			_Results = ResultsList;
			_bTree = bTree;
			this.GenerateList();
			if (_Results != null)
				_Results.CollectionChanged += CollectionChanged;
		}

		public void ItemSelected(IdentifyResult id)
		{
			_iIndex = _ResultList.IndexOf(id);
			this.RaiseCanExecuteChanged();
		}

		// Event handlers

		void CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
		{
			this.GenerateList();
			if (_ResultList.Count < 2)
				_iIndex = -1;
			else
				_iIndex = 0; // Assume that the first item will always be selected
			this.RaiseCanExecuteChanged();
		}

		// Private methods

		private void RaiseCanExecuteChanged()
		{
			if (this.CanExecuteChanged == null)
				return;
			this.CanExecuteChanged(this, new EventArgs());
		}

		private void GenerateList()
		{
			_ResultList = new List<IdentifyResult>();
			foreach (IdentifyResult id in _Results)
			{
				if (_bTree)
					this.ProcessTreeItem(id);
				else
					_ResultList.Add(id);
			}
		}

		private void ProcessTreeItem(IdentifyResult id)
		{
			if (id.Feature != null)
				_ResultList.Add(id);
			if (id.RelatedRecords == null)
				return;
			foreach (IdentifyResult idRelated in id.RelatedRecords)
				this.ProcessTreeItem(idRelated);
		}

	}

}
