﻿using System;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Input;

using Esri.ArcGISRuntime.Data;
using Extensibility;

namespace QueryAddin
{

	public interface IAttachmentDialogModel
	{
		void Initialize(IdentifyResult idResult);
	}

	class BasicAttachmentDialogModel : IAttachmentDialogModel, ICommand, IDialogContent
	{

		private int _iSelectedIndex;

		// Bindable properties

		public ObservableCollection<AttachmentDataItem> AttachmentItems { get; private set; }

		public int SelectedIndex
		{
			get { return _iSelectedIndex; } 
			set
			{
				_iSelectedIndex = value;
				if (this.CanExecuteChanged == null)
					return;
				this.CanExecuteChanged(this, new EventArgs());
			}
		}

		public ICommand Command { get { return this; } }

		public Visibility EditVisibility { get { return Visibility.Collapsed; } }

		// IAttachmentDialogModel

		public void Initialize(IdentifyResult idResult)
		{
			AttachmentItems = new ObservableCollection<AttachmentDataItem>();
			foreach (Attachment item in idResult.Attachments)
				AttachmentItems.Add(new AttachmentDataItem(item));
			//SelectedIndex = -1;
		}

		// ICommand

		public bool CanExecute(object parameter)
		{
			string sParam = parameter as string;
			switch (sParam)
			{
				case "Open":
				case "SaveAs":
					return (SelectedIndex > -1);
				case "SaveAll":
				case "OK":
				case "Cancel":
					return true;
			}
			return false;
		}

		public event EventHandler CanExecuteChanged;

		public void Execute(object parameter)
		{
			AttachmentDataItem item;
			string sParam = parameter as string;
			switch (sParam)
			{
				case "Open":
					item = AttachmentItems[SelectedIndex];
					item.Open();
					return;
				case "SaveAs":
					item = AttachmentItems[SelectedIndex];
					item.Save();
					MapApplication.Current.Message("Attachment saved.");
					return;
				case "SaveAll":
					string sFolder = DialogHelper.GetFolder();
					if (string.IsNullOrEmpty(sFolder))
						return;
					foreach (AttachmentDataItem it in AttachmentItems)
						it.SaveToFolder(sFolder);
					MapApplication.Current.Message("Attachment(s) saved.");
					return;
				case "OK":
					if (this.DialogOK == null)
						return;
					this.DialogOK(this, new EventArgs());
					return;
				case "Cancel":
					if (this.DialogCancel == null)
						return;
					this.DialogCancel(this, new EventArgs());
					return;
			}
		}

		// IDialogContent

		public string Title
		{
			get { return "Attachments"; }
		}

		public bool Closable
		{
			get { return true; }
		}

		public event EventHandler<EventArgs> DialogOK;

		public event EventHandler<EventArgs> DialogCancel;

	}

}
