﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Windows.Input;

using Esri.ArcGISRuntime.Data;

namespace QueryAddin
{

	// IdentifyResult supports both tree-style and list-style collections

	public struct Attribute
	{
		public string Key { get; set; }
		public string Value { get; set; }
	}

	public class IdentifyResult : INotifyPropertyChanged, ICommand
	{

		private bool _bExpanded = false;
		private bool _bSelected = false;

		// Bindable properties

		public string Value { get; set; }

		public bool IsExpanded
		{
			get { return _bExpanded; }
			set
			{
				_bExpanded = value;
				this.RaisePropertyChanged();
			}
		}

		public bool IsSelected
		{
			get { return _bSelected; }
			set
			{
				_bSelected = value;
				this.RaisePropertyChanged();
				this.RaiseIsSelectedChanged();
			}
		}

		public ICommand ContextMenuCommand { get { return this; } }

		public ObservableCollection<Attribute> Attributes { get; set; }

		public ObservableCollection<Attachment> Attachments { get; set; }

		public ObservableCollection<IdentifyResult> RelatedRecords { get; set; }

		public Feature Feature { get; set; }

		public string LayerID { get; set; }

		public IdentifyResult Parent { get; set; }

		// INotifyPropertyChanged

		public event PropertyChangedEventHandler PropertyChanged;

		// ICommand

		public bool CanExecute(object parameter)
		{
			if (Feature == null)
				return false;
			if (Feature.Geometry == null)
				return false;
			if (Feature.Geometry.IsEmpty)
				return false;
			return true;
		}

		public event EventHandler CanExecuteChanged { add { } remove { } } // Not used

		public void Execute(object parameter)
		{
			this.ExecuteContextMenu(parameter as string);
		}

		// Public events

		public event EventHandler IsSelectedChanged;

		// Private methods

		private void RaisePropertyChanged([CallerMemberName]string sPropertyName = null)
		{
			if (this.PropertyChanged == null)
				return;
			this.PropertyChanged(this, new PropertyChangedEventArgs(sPropertyName));
		}

		private void RaiseIsSelectedChanged()
		{
			if (this.IsSelectedChanged == null)
				return;
			this.IsSelectedChanged(this, new EventArgs());
		}

		private void ExecuteContextMenu(string sAction)
		{
			if (!this.CanExecute(null))
				return;
			FlashParameters fp = new FlashParameters();
			fp.Geometry = Feature.Geometry;
			fp.Flash = true;
			fp.ShowFlashLines = true;
			switch (sAction)
			{
				case "ZoomTo":
					fp.Navigate = FlashNavigation.Zoom;
					break;
				case "PanTo":
					fp.Navigate = FlashNavigation.Pan;
					break;
				case "Flash":
					fp.Navigate = FlashNavigation.None;
					break;
			}
			QueryViewExt.Current.Flash(fp);
		}

	}

}
