﻿using System;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Windows;
using System.Windows.Input;

using Brush = System.Windows.Media.Brush;
using FontFamily = System.Windows.Media.FontFamily;

namespace QueryAddin
{

	// MapTextModel acts as the data context for map text overlays

	public enum MapTextAction
	{
		UpdateLayout,
		Move,
		Delete
	}

	public class NotifyMapTextActionArgs : EventArgs
	{
		public MapTextAction Action;
	}

	public class MapTextModel : INotifyPropertyChanged, ICommand
	{

		public string ID = Guid.NewGuid().ToString();

		private string _sText;
		private FontFamily _FontFamily;
		private double _dFontSize;
		private FontWeight _FontWeight;
		private FontStyle _FontStyle;
		private TextDecorationCollection _TextDecorations;
		private TextAlignment _TextAlignment;
		private Brush _brushForeground;
		private double _dAngle = 0;
		private bool _bSelected = false;

		// Bindable properties

		public string MapTextValue
		{
			get { return _sText; }
			set
			{
				_sText = value;
				this.RaisePropertyChanged();
			}
		}

		public FontFamily MapTextFontFamily
		{
			get { return _FontFamily; }
			set
			{
				_FontFamily = value;
				this.RaisePropertyChanged();
			}
		}

		public double MapTextFontSize
		{
			get { return _dFontSize; }
			set
			{
				_dFontSize = value;
				this.RaisePropertyChanged();
			}
		}

		public FontWeight MapTextFontWeight
		{
			get { return _FontWeight; }
			set
			{
				_FontWeight = value;
				this.RaisePropertyChanged();
			}
		}

		public FontStyle MapTextFontStyle
		{
			get { return _FontStyle; }
			set
			{
				_FontStyle = value;
				this.RaisePropertyChanged();
			}
		}

		public TextDecorationCollection MapTextDecorations
		{
			get { return _TextDecorations; }
			set
			{
				_TextDecorations = value;
				this.RaisePropertyChanged();
			}
		}

		public TextAlignment MapTextAlignment
		{
			get { return _TextAlignment; }
			set
			{
				_TextAlignment = value;
				this.RaisePropertyChanged();
			}
		}

		public Brush MapTextForeground
		{
			get { return _brushForeground; }
			set
			{
				_brushForeground = value;
				this.RaisePropertyChanged();
			}
		}

		public double MapTextAngle
		{
			get { return _dAngle; }
			set
			{
				_dAngle = value;
				this.RaisePropertyChanged();
			}
		}

		public bool MapTextIsSelected
		{
			get { return _bSelected; }
			set
			{
				_bSelected = value;
				this.RaisePropertyChanged();
			}
		}

		public ICommand ContextCommand { get { return this; } }

		// INotifyPropertyChanged

		public event PropertyChangedEventHandler PropertyChanged; 

		// ICommand

		public bool CanExecute(object parameter) { return true; }

		public event EventHandler CanExecuteChanged { add { } remove { } } // Not used

		public void Execute(object parameter)
		{
			string sParam = parameter as string;
			if (sParam == null)
				return;
			switch (sParam)
			{
				case "Move":
					this.RaiseActionRequest(MapTextAction.Move);
					break;
				case "Delete":
					this.RaiseActionRequest(MapTextAction.Delete);
					break;
				case "OpenProperties":
					TextDialogModel.OpenExisting(this);
					break;
			}
		}

		// Public events

		public event EventHandler<NotifyMapTextActionArgs> ActionRequest;

		// Public methods

		public void UpdateLayout()
		{
			this.RaiseActionRequest(MapTextAction.UpdateLayout);
		}

		// Private methods

		private void RaisePropertyChanged([CallerMemberName]string sPropertyName = null)
		{
			if (this.PropertyChanged == null)
				return;
			this.PropertyChanged(this, new PropertyChangedEventArgs(sPropertyName));
		}

		private void RaiseActionRequest(MapTextAction action)
		{
			if (this.ActionRequest == null)
				return;
			NotifyMapTextActionArgs e = new NotifyMapTextActionArgs();
			e.Action = action;
			this.ActionRequest(this, e);
		}

	}
}
